import 'dart:async';
import 'package:args/command_runner.dart';
import 'package:calculator_cli/src/core/exist_codes.dart';
import 'package:mason_logger/mason_logger.dart';

class AddCommand extends Command<int> {
  AddCommand({
    required Logger logger,
  }) : _logger = logger;
  final Logger _logger;

  @override
  String get description => 'Add two or more than two number';

  @override
  String get name => 'add';

  @override
  String get usage => 'calculator_cli.dart add <a> <b> ..... <n>';

  @override
  FutureOr<int>? run() async {
    final inputs = argResults?.rest ?? [];
    final numbers = inputs
        .map((e) {
          final n = num.tryParse(e);
          if (n == null) {
            _logger.err('Invalid number $e');
          } else {
            _logger.detail('Valid number $n');
          }
          return n;
        })
        .whereType<num>()
        .toList();
    if (numbers.length != inputs.length) {
      _logger.err('One of many input is not a valid number');
      return ExitCodes.errors;
    }
    final progress = _logger.progress('Calculating');
    await Future.delayed(const Duration(seconds: 1));
    progress.complete('Calculation completed');
    final sum =
        numbers.fold(0.0, (previousValue, element) => previousValue + element);
    _logger.info('${numbers.join(' + ')} = $sum');
    return ExitCodes.success;
  }
}
