import 'dart:async';
import 'package:prompts/prompts.dart' as prompts;
import 'package:args/command_runner.dart';
import 'package:calculator_cli/src/core/exist_codes.dart';
import 'package:mason_logger/mason_logger.dart';

final allowedShape = [
  'square',
  'rectangle',
  'circle',
];

class AreaRunner extends Command<int> {
  AreaRunner({
    required Logger logger,
  }) : _logger = logger {
    argParser
      ..addOption(
        'length',
        abbr: 'l',
      )
      ..addOption(
        'breadth',
        abbr: 'b',
      )
      ..addOption(
        'radius',
        abbr: 'r',
      );
  }
  final Logger _logger;

  @override
  String get description => 'Find area of different shape';

  @override
  String get name => 'area';

  @override
  String get usage => 'calculator_cli.dart area ';

  @override
  FutureOr<int>? run() async {
    _logger.detail('AreaRunner Started');
    final shape = prompts.choose('Select a shape', allowedShape);
    if (!allowedShape.contains(shape)) {
      _logger.err('Shape is Required');
      return ExitCodes.errors;
    }
    num? result;
    switch (shape) {
      case 'square':
        result = _calculateCircleSquare();
        break;
      case 'rectangle':
        result = _calculateCircleRectange();
        break;
      case 'circle':
        result = _calculateCircleArea();
        break;
      default:
        _logger.err('$shape is not a valid shape for CLI');
    }
    if (result == null) {
      _logger.err('$shape is not a valid shape for CLI');
      return ExitCodes.errors;
    }

    _logger.detail('Calculating $shape area');
    return ExitCodes.success;
  }

  num _calculateCircleSquare() {
    final s = prompts.getDouble('Enter the side of Square');
    final result = s * s;
    _logger.info('Area of Square(s = $s) is ${result.toStringAsFixed(2)}');
    return result;
  }

  num _calculateCircleRectange() {
    final l = prompts.getDouble('Enter the length of Rectangle');
    final b = prompts.getDouble('Enter the breadth of Rectangle');
    final result = l * b;
    _logger.info(
      'Area of Rectangle(l = $l, b = $b) is ${result.toStringAsFixed(2)}',
    );
    return result;
  }

  num _calculateCircleArea() {
    final raidus = prompts.getDouble('Enter the radius of Circle');
    final result = (22 / 7) * raidus * raidus;
    _logger.info('Area of Circle(r = $raidus) is ${result.toStringAsFixed(2)}');
    return result;
  }
}
